
#include <linux/config.h>
#include <linux/module.h>
#include <linux/fs.h>
#include <linux/sysctl.h>
#include <linux/init.h>

#include <asm/uaccess.h>

MODULE_LICENSE("GPL");
int a[4];

/* checker function, uses minmax but takes hold of the values immediately */
int ssc_a_file(ctl_table *table, int write, struct file *filp,
                  void *buffer, size_t *lenp)
{
    int retval;

    if (!write) printk(KERN_INFO "ssc: a is being read\n");
    retval = proc_dointvec_minmax(table, write, filp, buffer, lenp);
    if (!write || retval < 0) return retval;
    printk(KERN_INFO "ssc: a is now %i, %i, %i, %i\n", a[0], a[1], a[2], a[3]);
    return retval;
}

#define DEV_SSC 45674 /* random */

ctl_table ssc_table[] = {
    {
        ctl_name:      1,
        procname:      "ints",
        data:          a,
        maxlen:        4*sizeof(int),
        mode:          0666,
        proc_handler:  proc_dointvec,
        strategy:      sysctl_intvec,
    },
    {0,}
};

static ctl_table dev_table[] = {
    {
        ctl_name:      DEV_SSC,
        procname:      "smallsysctl",
        mode:          0555,
	child:         ssc_table,
    },
    {0,}
};

static ctl_table root_table[] = {
    {
        ctl_name:      CTL_DEV,
        procname:      "dev",
        mode:          0555,
        child:         dev_table,
    },
    {0,}
};
  
static struct ctl_table_header *ssc_header;

int ssc_init(void)
{
    ssc_header = register_sysctl_table(root_table, 0);
    if (ssc_header) return 0;
    return -EBUSY;
}

void ssc_exit(void)
{
    unregister_sysctl_table(ssc_header);
}

module_init(ssc_init);
module_exit(ssc_exit);


